@@ -2,6 +2,7 @@
 # imports
 from syst3m.classes.config import *
 from syst3m.classes.color import color
+from syst3m.classes import defaults
 
 # input.
 _input_ = input
@@ -40,6 +41,8 @@ def execute(
 	arguments=[], 
 	# the subprocess shell parameter.
 	shell=False,
+	# asynchronous (cant capture output).
+	async_=False,
 	# serialize to dict.
 	# aka expect dict format.
 	serialize=False,
@@ -64,37 +67,31 @@ def execute(
 				dictionary[key] = value
 			ResponseObject.__init__(self, dictionary)
 			if self.success:
-				self.str = self["output"]
+				self.str = str(self.output)
 			else:
-				self.str = self.error
+				self.str = str(self.error)
 		def __len__(self):
 			return len(self.str)
 		def __str__(self):
 			return str(self.str)
+		def __bool__(self):
+			return bool(self.success)
 		# support default iteration.
 		def __iter__(self):
 			return iter(str(self))
 		# support '==' & '!=' operator.
 		def __eq__(self, dictionary):
-			if isinstance(dictionary, str):
-				return self.str == dictionary
-			elif isinstance(dictionary, String):
-				return self.str == dictionary.string
-			elif isinstance(dictionary, dict):
-				return str(self.sort()) == str(Files.Dictionary(dictionary).sort())
-			elif not isinstance(dictionary, (self.__class__, Dictionary)):
-				return False
-			return str(self.sort()) == str(dictionary.sort())
+			# new.
+			if isinstance(dictionary, (dict, Dictionary, Output, ResponseObject)):
+				return str(self.sort()) == str(dictionary.sort())
+			else:
+				return self.str == str(dictionary)
 		def __ne__(self, dictionary):
-			if isinstance(dictionary, str):
-				return self.str == dictionary
-			elif isinstance(dictionary, String):
-				return self.str == dictionary.string
-			elif isinstance(dictionary, dict):
-				return str(self.sort()) !=  str(Files.Dictionary(dictionary).sort())
-			elif not isinstance(dictionary, (self.__class__, Dictionary)):
-				return True
-			return str(self.sort()) != str(dictionary.sort())
+			# new.
+			if isinstance(dictionary, (dict, Dictionary, Output, ResponseObject)):
+				return str(self.sort()) != str(dictionary.sort())
+			else:
+				return self.str != str(dictionary)
 		# support 'in' operator.
 		def __contains__(self, string):
 			if isinstance(path, (list, Array)):
@@ -121,6 +118,15 @@ def execute(
 		response_str = f"command ({command})"
 	else:
 		response_str = f"script ({path})"
+	if async_:
+		proc = subprocess.Popen(
+			[executive, path] + arguments,
+			shell=shell,)
+		return Output(message=f"Succesfully executed {response_str}.", attributes={
+			"output":"",
+			"process":proc,
+			"pid":proc.pid,
+		})
 	try:
 		proc = subprocess.run(
 			[executive, path] + arguments,
@@ -137,12 +143,18 @@ def execute(
 		error_, output = error.stderr, error.output
 		if isinstance(error_, bytes): error_ = error_.decode()
 		if isinstance(output, bytes): output = output.decode()
-		return Output(error=f"Failed to execute {response_str}, (exception: {exception}), (output: {output}), (error: {error_}).")
+		if defaults.options.log_level <= 0:
+			return Output(error=f"Failed to execute {response_str}, (exception: {exception}), (error: {error_}).")
+		else:
+			return Output(error=f"Failed to execute {response_str}, (exception: {exception}), (output: {output}), (error: {error_}).")
 	error_, output = proc.stderr, proc.stdout
 	if isinstance(error_, bytes): error_ = error_.decode()
 	if isinstance(output, bytes): output = output.decode()
 	if error_ != "":
-		return Output(error=f"Failed to execute {response_str}, (output: {output}), (error: {error_}).")
+		if defaults.options.log_level <= 0:
+			return Output(error=f"Failed to execute {response_str}, (error: {error_}).")
+		else:
+			return Output(error=f"Failed to execute {response_str}, (output: {output}), (error: {error_}).")
 	if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
 	if delete:
 		Files.delete(path)
@@ -150,13 +162,13 @@ def execute(
 		try: response = r3sponse.ResponseObject(json=output)
 		except Exception as e: 
 			if loader != None: loader.stop(success=False)
-			return Output(error=f"Failed to serialize output: {output}")
+			return Output(error=f"Failed to serialize (output: {output}).")
 		return response
 	else:
-		return Output(message=f"Succesfully executed {response_str}, (output: {output}), (error: {error_}).", attributes={
+		return Output(message=f"Succesfully executed {response_str}.", attributes={
 			"output":output,
 			"process":proc,
-			#"pid":proc.pid, # only for subprocess.Popen
+			"pid":None,
 		})
 
 # the loader object class.
